"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __esm = (fn, res) => function __init() {
  return fn && (res = (0, fn[__getOwnPropNames(fn)[0]])(fn = 0)), res;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// src/types.ts
var ECMASyntaxError;
var init_types = __esm({
  "src/types.ts"() {
    "use strict";
    ECMASyntaxError = class extends Error {
      constructor(message, options) {
        super(message);
        __publicField(this, "source");
        __publicField(this, "output");
        this.source = options.source;
        this.output = options.output;
      }
    };
  }
});

// src/helpers/utils.ts
function checkIsExcludeSource(path, exclude) {
  if (!exclude) {
    return false;
  }
  const excludes = Array.isArray(exclude) ? exclude : [exclude];
  return excludes.some((reg) => reg.test(path));
}
var init_utils = __esm({
  "src/helpers/utils.ts"() {
    "use strict";
  }
});

// src/helpers/generateError.ts
function displayCodePointer(code, pos) {
  const SUB_LEN = 80;
  const start = Math.max(pos - SUB_LEN / 2, 0);
  const subCode = code.slice(start, start + SUB_LEN);
  const arrowPos = pos - start;
  const arrowLine = import_shared.color.yellow("^".padStart(arrowPos + 11, " "));
  return `${subCode}
${arrowLine}`;
}
async function generateError({
  err,
  code,
  filepath,
  rootPath,
  exclude
}) {
  let error = await tryGenerateErrorFromSourceMap({
    err,
    filepath,
    rootPath
  });
  if (!error) {
    const path = filepath.replace(rootPath, "");
    error = new ECMASyntaxError(err.message, {
      source: {
        path,
        line: err.loc.line,
        column: err.loc.column,
        code: displayCodePointer(code, err.pos)
      }
    });
  }
  if (checkIsExcludeSource(error.source.path, exclude)) {
    return null;
  }
  return error;
}
function makeCodeFrame(lines, highlightIndex) {
  const startLine = Math.max(highlightIndex - 3, 0);
  const endLine = Math.min(highlightIndex + 4, lines.length - 1);
  const ret = [];
  for (let i = startLine; i < endLine; i++) {
    if (i === highlightIndex) {
      const lineNumber = `> ${i + 1}`.padStart(6, " ");
      ret.push(import_shared.color.yellow(`${lineNumber} | ${lines[i]}`));
    } else {
      const lineNumber = ` ${i + 1}`.padStart(6, " ");
      ret.push(import_shared.color.gray(`${lineNumber} | ${lines[i]}`));
    }
  }
  return `
${ret.join("\n")}`;
}
async function tryGenerateErrorFromSourceMap({
  err,
  filepath,
  rootPath
}) {
  const sourceMapPath = `${filepath}.map`;
  if (!import_shared.fse.existsSync(sourceMapPath)) {
    return null;
  }
  try {
    const sourcemap = await import_shared.fse.readFile(sourceMapPath, "utf-8");
    const consumer = await new import_source_map.SourceMapConsumer(sourcemap);
    const sm = consumer.originalPositionFor({
      line: err.loc.line,
      column: err.loc.column
    });
    if (!sm.source) {
      return null;
    }
    const { sources } = consumer;
    const smIndex = sources.indexOf(sm.source);
    const smContent = JSON.parse(sourcemap)?.sourcesContent?.[smIndex];
    if (!smContent) {
      return null;
    }
    const path = sm.source.replace(/webpack:\/\/(tmp)?/g, "");
    const relativeFilepath = filepath.replace(rootPath, "");
    const rawLines = smContent.split(/\r?\n/g);
    const highlightLine = (sm.line ?? 1) - 1;
    return new ECMASyntaxError(err.message, {
      source: {
        path,
        line: sm.line ?? 0,
        column: sm.column ?? 0,
        code: makeCodeFrame(rawLines, highlightLine)
      },
      output: {
        path: relativeFilepath,
        line: err.loc.line,
        column: err.loc.column
      }
    });
  } catch (e) {
    return null;
  }
}
var import_shared, import_source_map;
var init_generateError = __esm({
  "src/helpers/generateError.ts"() {
    "use strict";
    import_shared = require("@rsbuild/shared");
    import_source_map = require("source-map");
    init_types();
    init_utils();
  }
});

// src/helpers/printErrors.ts
function printErrors(errors) {
  if (errors.length === 0) {
    import_core.logger.success("The syntax check success.");
    return;
  }
  const errs = errors.map((error) => ({
    source: `${error.source.path}:${error.source.line}:${error.source.column}`,
    output: error.output ? `${error.output.path}:${error.output.line}:${error.output.column}` : void 0,
    reason: error.message,
    code: error.source.code
  }));
  const longest = Math.max(...Object.keys(errs[0]).map((err) => err.length));
  import_core.logger.error(
    "[Syntax Checker] Find some syntax errors after production build:\n"
  );
  errs.forEach((err, index) => {
    console.log(import_shared2.color.bold(import_shared2.color.red(`  ERROR ${index + 1}`)));
    printMain(err, longest);
  });
  throw new Error(
    `[Syntax Checker] The current build fails due to an incompatible syntax, which can be fixed in the following ways:

  - If you want to downgrade the syntax, you can compile the corresponding module through the \`source.include\` config.
  - If you don't want to downgrade the syntax, you can adjust the project's browserslist to match the syntax.`
  );
}
function printMain(error, longest) {
  const fillWhiteSpace = (s, longest2) => {
    if (s.length < longest2) {
      const rightPadding = " ".repeat(longest2 - s.length);
      return s + rightPadding;
    }
    return s;
  };
  for (const [key, content] of Object.entries(error)) {
    if (!content) {
      continue;
    }
    const title = import_shared2.color.magenta(`${fillWhiteSpace(`${key}:`, longest + 1)}`);
    console.info(`  ${title}  ${content}`);
  }
  console.info("");
}
var import_core, import_shared2;
var init_printErrors = __esm({
  "src/helpers/printErrors.ts"() {
    "use strict";
    import_core = require("@rsbuild/core");
    import_shared2 = require("@rsbuild/shared");
  }
});

// src/helpers/generateHtmlScripts.ts
async function generateHtmlScripts(filepath) {
  const html = await import_shared3.fse.readFile(filepath, "utf-8");
  return getHtmlScripts(html);
}
function getHtmlScripts(html) {
  const inlineScripts = [];
  let currentScript = null;
  const parser = new import_htmlparser2.Parser({
    onopentag(name, attrs) {
      if (name === "script" && !attrs.src && (!attrs.type || attrs.type === "application/javascript" || attrs.type === "text/javascript")) {
        currentScript = "";
      }
    },
    ontext(text) {
      if (currentScript !== null) {
        currentScript += text;
      }
    },
    onclosetag(tagname) {
      if (tagname === "script" && currentScript) {
        inlineScripts.push(currentScript.trim());
      }
      currentScript = null;
    }
  });
  parser.write(html);
  parser.end();
  return inlineScripts;
}
var import_shared3, import_htmlparser2;
var init_generateHtmlScripts = __esm({
  "src/helpers/generateHtmlScripts.ts"() {
    "use strict";
    import_shared3 = require("@rsbuild/shared");
    import_htmlparser2 = require("htmlparser2");
  }
});

// src/helpers/index.ts
var init_helpers = __esm({
  "src/helpers/index.ts"() {
    "use strict";
    init_generateError();
    init_printErrors();
    init_generateHtmlScripts();
    init_utils();
  }
});

// src/CheckSyntaxPlugin.ts
var CheckSyntaxPlugin_exports = {};
__export(CheckSyntaxPlugin_exports, {
  CheckSyntaxPlugin: () => CheckSyntaxPlugin
});
var import_node_path, import_shared4, import_acorn, CheckSyntaxPlugin;
var init_CheckSyntaxPlugin = __esm({
  "src/CheckSyntaxPlugin.ts"() {
    "use strict";
    import_node_path = require("path");
    import_shared4 = require("@rsbuild/shared");
    import_acorn = require("acorn");
    init_helpers();
    CheckSyntaxPlugin = class _CheckSyntaxPlugin {
      constructor(options) {
        __publicField(this, "errors", []);
        __publicField(this, "ecmaVersion");
        __publicField(this, "targets");
        __publicField(this, "rootPath");
        __publicField(this, "exclude");
        this.targets = options.targets;
        this.exclude = options.exclude;
        this.rootPath = options.rootPath;
        this.ecmaVersion = options.ecmaVersion || (0, import_shared4.browserslistToESVersion)(this.targets);
      }
      apply(complier) {
        complier.hooks.afterEmit.tapPromise(
          _CheckSyntaxPlugin.name,
          async (compilation) => {
            const outputPath = compilation.outputOptions.path || "dist";
            const emittedAssets = compilation.getAssets().filter((a) => a.source).map((a) => {
              const resourcePath = a.name.split("?")[0];
              return (0, import_node_path.resolve)(outputPath, resourcePath);
            });
            const files = emittedAssets.filter(
              (assets) => import_shared4.HTML_REGEX.test(assets) || import_shared4.JS_REGEX.test(assets)
            );
            await Promise.all(
              files.map(async (file) => {
                await this.check(file);
              })
            );
            printErrors(this.errors);
          }
        );
      }
      async check(filepath) {
        if (import_shared4.HTML_REGEX.test(filepath)) {
          const htmlScripts = await generateHtmlScripts(filepath);
          await Promise.all(
            htmlScripts.map(async (script) => {
              if (!checkIsExcludeSource(filepath, this.exclude)) {
                await this.tryParse(filepath, script);
              }
            })
          );
        }
        if (import_shared4.JS_REGEX.test(filepath)) {
          const jsScript = await import_shared4.fse.readFile(filepath, "utf-8");
          await this.tryParse(filepath, jsScript);
        }
      }
      async tryParse(filepath, code) {
        try {
          (0, import_acorn.parse)(code, { ecmaVersion: this.ecmaVersion });
        } catch (_) {
          const err = _;
          const error = await generateError({
            err,
            code,
            filepath,
            exclude: this.exclude,
            rootPath: this.rootPath
          });
          if (error) {
            this.errors.push(error);
          }
        }
      }
    };
  }
});

// src/index.ts
var src_exports = {};
__export(src_exports, {
  pluginCheckSyntax: () => pluginCheckSyntax
});
module.exports = __toCommonJS(src_exports);
var import_shared5 = require("@rsbuild/shared");
async function getTargets(rootPath, rsbuildConfig, rsbuildTarget, checkSyntax) {
  return checkSyntax.targets ?? await (0, import_shared5.getBrowserslistWithDefault)(rootPath, rsbuildConfig, rsbuildTarget);
}
function pluginCheckSyntax(options = {}) {
  return {
    name: "rsbuild:check-syntax",
    setup(api) {
      api.modifyBundlerChain(async (chain, { isDev, target }) => {
        if (isDev || target !== "web") {
          return;
        }
        const { rootPath } = api.context;
        const rsbuildConfig = api.getNormalizedConfig();
        const targets = await getTargets(
          rootPath,
          rsbuildConfig,
          target,
          options
        );
        const { CheckSyntaxPlugin: CheckSyntaxPlugin2 } = await Promise.resolve().then(() => (init_CheckSyntaxPlugin(), CheckSyntaxPlugin_exports));
        chain.plugin(CheckSyntaxPlugin2.name).use(CheckSyntaxPlugin2, [
          {
            targets,
            rootPath,
            ...typeof options === "object" ? options : {}
          }
        ]);
      });
    }
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  pluginCheckSyntax
});
